<div class="panel panel-default llcontainer">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <div class="panel-body">
    <div class="form-group">
      <label for="filetext" id="lft"><span trspan="editContent"></span> :</label>
      <textarea id="filetext" aria-describedby="lft" class="form-control" rows="8" ng-model="currentNode.data"></textarea>
    </div>
    <div class="form-group">
      <label for="fileinput" id="lfi"><span trspan="replaceByFile"></span> :</label>
      <input id="fileinput" aria-describedby="lfi" type="file" class="form-control" on-read-file="replaceContent(currentNode,$fileContent)"/>
    </div>
    <div class="form-group">
      <label for="urlinput" id="lfu"><span trspan="loadFromUrl"></span> :</label>
      <span class="input-group">
        <input id="urlinput" aria-describedby="lfu" class="form-control" ng-model="url" />
        <span id="urlload" class="input-group-addon link" trspan="load" ng-click="replaceContentByUrl(currentNode,url)"></span>
      </span>
    </div>
  </div>
</div>
<script type="text/menu">
[{
  "title": "cancel",
  "icon": "remove"
},{
  "title": "download",
  "icon": "save-file",
  "action": "saveAsText"
}]
</script>
