<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <div class="panel-body">
    <div class="input-group-solid" role="radiogroup">
      <label class="radio-inline">
        <input id="bopeOn" type="radio" ng-value="1" name="bope" ng-model="currentNode.data" ng-checked="currentNode.data==1 || currentNode.data=='1'" aria-labelledby="onl" role="radio"/>
        <label id="onl" for="bopeOn" trspan="on"></label>
      </label>
      <label class="radio-inline">
        <input id="bopeOff" type="radio" ng-value="0" name="bope" ng-model="currentNode.data" ng-checked="currentNode.data!==1&&currentNode.data!=='1'" aria-labelledby="offl" role="radio"/>
        <label id="offl" for="bopeOff" trspan="off"></label>
      </label>
    </div>
  </div>
</div>
<script type="text/menu">
[{
  "title": "cancel",
  "icon": "remove"
}]
</script>
