<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <table class="table table-striped">
    <tbody>
      <tr ng-repeat="n in currentNode.nodes">
        <th>{{translateTitle(n)}}</th>
        <!-- TEXT -->
        <td ng-if="!n.type||n.type=='text'">
          <input class="form-control" ng-model="n.data" />
        </td>
        <!-- PASSWORD -->
        <td ng-if="n.type=='password'">
          <input type="password" class="form-control" ng-model="n.data" />
        </td>
        <!-- LONG TEXT -->
        <td ng-if="n.type=='longtext'">
          <div class="input-group maxw">
            <textarea rows="2" ng-model="n.data"></textarea>
          </div>
        </td>
        <!-- INT -->
        <td ng-if="n.type=='int'">
          <input type="number" class="form-control" ng-model="n.data" />
        </td>
        <!-- BOOL -->
        <td ng-if="n.type=='bool'">
          <div class="input-group-solid" role="radiogroup">
            <label class="radio-inline">
              <input id="bopeOn/{{n.title}}" type="radio" ng-value="1" ng-model="n.data" ng-checked="n.data==1||n.data=='1'" role="radio" aria-labelledby="lbopeOn{{n.title}}"/>
              <span id="lbopeOn{{n.title}}" for="bopeOn/{{n.title}}" trspan="on"></span>
            </label>
            <label class="radio-inline">
              <input id="bopeOff/{{n.title}}" type="radio" ng-value="0" ng-model="n.data" ng-checked="n.data!==1&&n.data!=='1'" role="radio" aria-labelledby="lbopeOff{{n.title}}"/>
              <span id="lbopeOff{{n.title}}" for="bopeOff/{{n.title}}" trspan="off"></span>
            </label>
          </div>
        </td>
        <!-- TROOL -->
        <td ng-if="n.type=='trool'">
          <div class="input-group-solid" role="radiogroup">
            <label class="radio-inline">
              <input id="trOn/{{n.title}}" type="radio" ng-value="1" ng-model="n.data" role="radio" aria-labelledby="ltrOn{{n.title}}">
              <span id="ltrOn{{n.title}}" for="trOn/{{n.title}}" trspan="on"></span>
            </label>
            <label class="radio-inline">
              <input id="trOff/{{n.title}}" type="radio" ng-value="0" ng-model="n.data" role="radio" aria-labelledby="ltrOff{{n.title}}">
              <span id="ltrOff{{n.title}}" for="trOff/{{n.title}}" trspan="off"></span>
            </label>
            <label class="radio-inline">
              <input id="trDefault/{{n.title}}" type="radio" ng-value="-1" ng-model="n.data" ng-checked="n.data!==1&&n.data!=='1'&&n.data!==0&&n.data!=='0'" role="radio" aria-labelledby="ltrDefault{{n.title}}">
              <span id="ltrDefault{{n.title}}" for="trDefault/{{n.title}}" trspan="default"></span>
            </label>
          </div>
        </td>
        <!-- BOOL or EXPR -->
        <td ng-if="n.type=='boolOrExpr'">
          <div class="input-group-solid" role="radiogroup">
            <label class="radio-inline">
              <input id="boeOn/{{n.title}}" type="radio" ng-value="1" ng-model="n.data" role="radio" aria-labelledby="lboeOn{{n.title}}">
              <span id="lboeOn{{n.title}}" for="boeOn/{{n.title}}" trspan="on"></span>
            </label>
            <label class="radio-inline">
              <input id="boeOff/{{n.title}}" type="radio" ng-value="0" ng-model="n.data" role="radio" aria-labelledby="lboeOff{{n.title}}">
              <span id="lboeOff{{n.title}}" for="boeOff/{{n.title}}" trspan="off"></span>
            </label>
            <label class="radio-inline">
              <input id="boeExpr/{{n.title}}" type="radio" ng-value="-1" ng-click="n.data=''" ng-checked="n.data!==0&&n.data!=='0'&&n.data!==1&&n.data!=='1'" role="radio" aria-labelledby="lboeExpr{{n.title}}">
              <span id="lboeExpr{{n.title}}" for="boeExpr/{{n.title}}" trspan="specialRule"></span>
            </label>
          </div>
          <div class="panel-body input-group maxw" ng-if="n.data!==0&&n.data!=='0'&&n.data!==1&&n.data!=='1'">
            <textarea id="boeValue/{{n.title}}" class="form-control" rows="2" ng-model="n.data"></textarea>
          </div>
        </td>
        <!-- SELECT -->
        <td ng-if="n.type=='select'">
          <select class="form-control" ng-model="n.data">
            <option ng-repeat="item in n.select" ng-selected="item.k==n.data" value="{{item.k}}" trspan="{{item.v}}"></option>
          </select>
        </td>
        <th ng-if="n.type&&n.type!='text'&&n.type!='longtext'&&n.type!='int'&&n.type!='bool'&&n.type!='trool'&&n.type!='boolOrExpr'&&n.type!='select'&&n.type!='password'">
          ERROR, complex node inside. Don't use simpleInputContainer for {{currentNode.title}}
        </th>
      </tr>
    </tbody>
  </table>
<script type="text/menu">
[]
</script>
