// Generated by CoffeeScript 1.12.8

/*
LemonLDAP::NG U2F verify script
 */

(function() {
  var check;

  check = function() {
    return u2f.sign(window.datas.appId, window.datas.challenge, window.datas.registeredKeys, function(data) {
      $('#verify-data').val(JSON.stringify(data));
      $('#verify-challenge').val(window.datas.challenge);
      return $('#verify-form').submit();
    });
  };

  $(document).ready(function() {
    return setTimeout(check, 1000);
  });

}).call(this);
