<tr>
 <td>
  <iframe src="<TMPL_VAR NAME="url">" alt="" marginwidth="0"  marginheight="0" scrolling="no" class="noborder" width="10px" height="10px" frameborder="0"></iframe>
 </td>
 <td>
  <TMPL_VAR NAME="name">
 </td>
</tr>
