<html>
<body>

<div id="page" style="font-family:sans-serif;font-size:12pt;padding:5px 20px;">

<div id="header" style="text-align:center;"><img src="cid:logo:../../htdocs/static/<TMPL_VAR NAME="MAIN_LOGO">" style="max-width:150px;" alt="logo"/></div>

<div id="content" style="padding:5px 10px;margin:10px 0;">
