// Generated by CoffeeScript 1.12.8
(function() {
  $(document).ready(function() {
    return $(".idploop").on('click', function() {
      return $("#idp").val($(this).attr("val"));
    });
  });

}).call(this);
