/* CSS for bootstrap theme */

html,
body {
  height: 100%;
  background: radial-gradient(circle at 50% 0% ,  #fff 0%, #ddd 100%) no-repeat scroll 0% 0% #ddd;
  /* The html and body elements cannot have any padding or margin. */
}

/* Wrapper for page content to push down footer */
#wrap {
  min-height: 100%;
  height: auto;
  /* Negative indent footer by its height */
  margin: 0 auto -80px;
  /* Pad bottom by footer height */
  padding: 20px 0 80px;
}

/* Set the fixed height of the footer here */
#footer {
  height: 80px;
  background-color: rgb(255,255,255);
  background-color: rgba(255,255,255,0.9);
  text-align: center;
  padding-top: 10px;
  overflow: hidden;
}

#header img {
  background-color: rgb(255,255,255);
  background-color: rgba(255,255,255,0.8);
  margin-bottom: 20px;
}

.card, .navbar-light {
  background-color: rgb(255,255,255);
  background-color: rgba(255,255,255,0.9);
  background-image: none;
}

.login, .password {
  text-align: center;
  padding: 20px;
}

div.form {
  margin: 0 auto;
  max-width: 330px;
}

div.actions {
  margin: 10px 0 0 0;
}

div.actions a {
  margin-top: 10px;
}

.buttons {
  text-align: center;
  margin: 10px 0 0 0;
  cursor: pointer;
}
/* Override bootstrap btn style */
.btn {
  white-space: normal;
}

.btn span.fa {
  padding-right: 8px;
}

/* Make active tab more visible */
li.ui-state-active {
  background-color: rgb(250,250,250);
  background-color: rgba(250,250,250,0.9);
}

#appslist, #password, #loginHistory, #logout, #oidcConsents  {
  margin-top: 20px;
}

div.category {
  margin: 10px 0;
  cursor: grab;
}

div.application {
  margin: 5px 0;
  overflow: hidden;
}

div.application a, div.application a:hover {
  text-decoration:none;
}

p.notifCheck label {
  margin-left: 5px;
  margin-top: 3px;
  display: inline-block;
}

img.langicon {
  cursor: pointer;
}

button.idploop {
  max-width: 300px;
}

button.idploop img {
  max-height: 30px;
}

div.oidc_consent_message > ul {
  text-align: left;
  list-style: circle;
}

/* Responsive styles */
@media (min-width: 768px) {
  div.application {
    height: 80px;
  }
  div.application h4.appname {
    margin:0;
  }
  #wrap {
    margin: 0 auto -60px;
  }
  #footer {
    height: 60px;
  }
}

.hiddenFrame {
  border: none;
  display: hidden;
  margin: 0
}

.noborder {
  border: none;
}

.max {
  width: 100%;
}

.link {
  cursor: pointer;
}

.nodecor:hover, .nodecor:active, .nodecor:focus {
  text-decoration: none;
}

.fa.icon-blue {
    color: blue;
}

.progress-bar-animated {
  width: 100%;
}

input.key {
  font-family: 'password';
  width: 100px;  
}

@font-face {
  font-family: 'password';
  /*font-style: normal;*/
  /*font-weight: 400;*/
  src: url(/static/common/fonts/password.ttf);
}

.info.table caption {
  color: black;
  text-align: center;
  caption-side: bottom;
}

/* Allow full height for check box label */
div.input-group > p.form-control {
  height: auto;
}
div.input-group > p.form-control > label {
  display: revert;
  user-select: none;
  cursor: pointer;
}
