// Generated by CoffeeScript 1.12.8
(function() {
  var values;

  values = {};

  $(document).ready(function() {
    $("script[type='application/init']").each(function() {
      var e, k, results, tmp;
      try {
        tmp = JSON.parse($(this).text());
        results = [];
        for (k in tmp) {
          results.push(values[k] = tmp[k]);
        }
        return results;
      } catch (error) {
        e = error;
        return console.log('Parsing error', e);
      }
    });
    return window.addEventListener("message", function(e) {
      var client_id, message, salt, session_state, ss, stat;
      message = e.data;
      console.log('message=', message);
      client_id = decodeURIComponent(message.split(' ')[0]);
      session_state = decodeURIComponent(message.split(' ')[1]);
      salt = decodeURIComponent(session_state.split('.')[1]);
      ss = btoa(client_id + ' ' + e.origin + ' ' + salt) + '.' + salt;
      if (session_state === ss) {
        stat = 'unchanged';
      } else {
        stat = 'changed';
      }
      return e.source.postMessage(stat, e.origin);
    }, false);
  });

}).call(this);
