<html>
<head>
    <title>LemonLDAP::NG Manager API</title>
    <link rel="stylesheet" type="text/css" href="/static/bwr/bootstrap/dist/css/bootstrap.min.css"/>
</head>
<body>
    <div class="container text-center">
        <h1>LemonLDAP::NG Manager API</h1>
        <hr />
        <a href="/doc/pages/manager-api/index.html" class="btn btn-lg btn-primary">API Reference</a>
    </div>
</body>
</html>

