// Generated by CoffeeScript 1.12.8
(function() {
  var go;

  $(document).ready(function() {
    if (window.requestIdleCallback) {
      return requestIdleCallback(function() {
        return go();
      });
    } else {
      return setTimeout(go, 500);
    }
  });

  go = function() {
    return Fingerprint2.get(function(components) {
      var result, values;
      values = components.map((function(_this) {
        return function(component) {
          return component.value;
        };
      })(this));
      result = Fingerprint2.x64hash128(values.join(''), 31);
      $('#fg').attr("value", result);
      return $('#form').submit();
    });
  };

}).call(this);
