// Generated by CoffeeScript 1.12.8
(function() {
  $(document).ready(function() {
    return $.ajax(portal + '?kerberos=1', {
      dataType: 'json',
      statusCode: {
        401: function() {
          return $('#lformKerberos').submit();
        }
      },
      success: function(data) {
        return $('#lformKerberos').submit();
      },
      error: function() {
        return $('#lformKerberos').submit();
      }
    });
  });

}).call(this);
