<TMPL_INCLUDE NAME="mail_header.tpl">

<span>
<span trspan="hello">Hello</span> $cn,<br />
<br />
<span trspan="yourLoginCodeIs">Your login code is</span>
<b>$code</b><br/>
</span>

<TMPL_INCLUDE NAME="mail_footer.tpl">
